use nom::error::VerboseError;
use nom::IResult;

use super::list::List;

type Matcher = dyn for<'s> Fn(&'s str) -> IResult<&'s str, &'s str, VerboseError<&'s str>>;

#[derive(Debug, Clone)]
pub struct ContextTree<'r> {
    tree: List<ContextElement<'r>>,
}

impl<'r> ContextTree<'r> {
    pub fn new() -> Self {
        ContextTree { tree: List::new() }
    }

    pub fn with_additional_node<'x>(&self, data: ContextElement<'x>) -> ContextTree<'x> {
        let new_list = self.tree.push_front(data);
        ContextTree { tree: new_list }
    }

    pub fn check_fail_matcher<'s>(&'r self, i: &'s str) -> IResult<&'s str, &'s str, VerboseError<&'s str>> {
        // todo
        todo!()
    }
}

#[derive(Debug, Clone)]
pub enum ContextElement<'r> {
    FailMatcherNode(FailMatcherNode<'r>),
    PreviousElementNode(PreviousElementNode<'r>),
}

#[derive(Debug, Clone)]
pub struct FailMatcherNode<'r> {
    pub fail_matcher: ChainBehavior<'r>,
}

#[derive(Debug, Clone)]
pub struct PreviousElementNode<'r> {
    pub dummy: &'r str,
}

#[derive(Clone)]
pub enum ChainBehavior<'r> {
    AndParent(Option<&'r Matcher>),
    IgnoreParent(Option<&'r Matcher>),
}

impl<'r> std::fmt::Debug for ChainBehavior<'r> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let mut formatter = f.debug_struct("ChainBehavior");
        // match self {
        //     ChainBehavior::AndParent(_) => {
        //         formatter = formatter.field("type", &"AndParent");
        //     }
        //     ChainBehavior::IgnoreParent(_) => {
        //         formatter = formatter.field("type", &"IgnoreParent");
        //     }
        // };
        formatter.finish()
    }
}
